import os
import pandas as pd
import numpy as np
from context import demographicsGraphs
from context import vitalsGraphs
from context import vitalsAnomalies
from context import labMeasurementsGraphs
from context import anomaly
from datetime import date

import unittest

import dbUtils

class TestPlots(unittest.TestCase):


    def testDemographicsPlots(self):
        if os.path.exists('./demographics.html'):
            os.remove('./demographics.html')
        data = [
            [0, 1, 2, 'male', 'white', date.fromisoformat('2020-09-13'), date.fromisoformat('2021-09-13')], 
            [2, 3, 4, np.nan, 'white', date.fromisoformat('2020-09-14'), date.fromisoformat('2021-09-13')], 
            [4, 5, 6, 'female', 'black', date.fromisoformat('2020-09-15'), date.fromisoformat('2021-09-13')], 
            [6, 7, 8, np.nan, 'asian', date.fromisoformat('2020-09-14'), date.fromisoformat('2021-09-13')]]
        demographicsGraphs.plot(pd.DataFrame(data, columns=['age', 'weight', 'height', 'gender', 'ethnicity', 'dob', 'dod']))
        assert os.path.exists('./demographics.html')


    def testDemographicsMimicPlots(self):
        if os.path.exists('./demographics.html'):
            os.remove('./demographics.html')
        df = dbUtils._getDemographics()
        demographicsGraphs.plot(df)
        assert os.path.exists('./demographics.html')


    def testVitalsPlots(self):
        if os.path.exists('./vitals.html'):
            os.remove('./vitals.html')
        data = [
            [0, 1, 2], 
            [2, np.nan, 4], 
            [4, 5, np.nan], 
            [0, 1, 2], 
            [2, 3, 4], 
            [4, 5, np.nan], 
            [0, 1, 2], 
            [2, 3, 4], 
            [4, 5, 6], 
            [6, 7, np.nan]]
        vitalsGraphs.plot(pd.DataFrame(data, columns=['heartrate', 'sysbp', 'diabp']))
        assert os.path.exists('./vitals.html')


    def testVitalsMimicPlots(self):
        if os.path.exists('./vitals.html'):
            os.remove('./vitals.html')
        df = dbUtils._getVitals()
        vitalsGraphs.plot(df)
        assert os.path.exists('./vitals.html')


    def testLabMeasurementsPlots(self):
        if os.path.exists('./lab_measurements.html'):
            os.remove('./lab_measurements.html')
        data = [
            [0, 1, 2], 
            [2, np.nan, 4], 
            [4, 5, np.nan], 
            [0, 1, 2], 
            [2, 3, 4], 
            [4, 5, np.nan], 
            [0, 1, 2], 
            [2, 3, 4], 
            [4, 5, 6], 
            [6, 7, np.nan]]
        labMeasurementsGraphs.plot(pd.DataFrame(data, columns=['glucose', 'hemoglobin', 'anion_gap']))
        assert os.path.exists('./lab_measurements.html')


    def testLabMeasurementsMimicPlots(self):
        if os.path.exists('./lab_measurements.html'):
            os.remove('./lab_measurements.html')
        df = dbUtils._getLabMeasurements()
        labMeasurementsGraphs.plot(df)
        assert os.path.exists('./lab_measurements.html')


    def testAnomalies(self):
        data = [
            [1, 2], 
            [2, 3], 
            [3, 4], 
            [4, 5], 
            [5, 6], 
            [6, 7], 
            [7, 8], 
            [8, 9]]
        df = pd.DataFrame(data, columns=['x', 'y'])
        out = anomaly.irt_ensemble(df)
        assert out.x.sum() == 36


    def testVitalsAnomalies(self):
        if os.path.exists('./vitals_anomalies.html'):
            os.remove('./vitals_anomalies.html')
        df = dbUtils._getVitals()
        vitalsAnomalies.plot(df)
        assert os.path.exists('./vitals_anomalies.html')



import base64
import numpy as np
from io import BytesIO
from matplotlib import pyplot as plt
from yattag import Doc

from . import utils


doc, tag, text = Doc().tagtext()


def plot(
    df,
    outputFile = 'vitals.html',
    heartrateCol = 'heartrate',
    sysbpCol = 'sysbp',
    diabpCol = 'diabp',
    meanbpCol = 'meanbp',
    resprateCol = 'resprate',
    tempcCol = 'tempc',
    spo2Col = 'spo2',
    gcseyeCol = 'gcseye',
    gcsverbalCol = 'gcsverbal',
    gcsmotorCol = 'gcsmotor'):

    colNames = [heartrateCol, sysbpCol, diabpCol, meanbpCol, resprateCol, tempcCol, spo2Col, gcseyeCol, gcsverbalCol, gcsmotorCol]

    doc.asis('<!DOCTYPE html>')
    with tag('html'):
        doc.asis('<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.1/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-F3w7mX95PdgyTmZZMECAngseQB83DfGTowi0iMjiWaeVhAn4FJkqJByhZMI3AhiU" crossorigin="anonymous">')
        with tag('body'):
            doc.asis('<div style="clear:both;"></div>')
            with tag('div'):
                with tag('h1'):
                    doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                    with tag('span', klass='fs-4', style="margin: 10px;"):
                        text('Vitals Summary')
                __drawVitalsSummary(df, colNames)
            doc.asis('<div style="clear:both;"></div>')
            for col in colNames:
                if col in df.columns:
                    with tag('div'):
                        with tag('h1'):
                            doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                            with tag('span', klass='fs-4', style="margin: 10px;"):
                                text('Histogram - ' + col)
                        with tag('div', klass='col-5', style="float: left;"):
                            doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(__drawVitalsHistogram(df, col)))
                        with tag('div', klass='col-5', style="float: left;"):
                            doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(__drawVitalsBoxplot(df, col)))
                        with tag('div', klass='col-2', style="float: left;"):
                            __drawVitalsTable(df, col)
                doc.asis('<div style="clear:both;"></div>')
            with tag('div'):
                with tag('h1'):
                    doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                    with tag('span', klass='fs-4', style="margin: 10px;"):
                        text('Missing Data Plot')
                doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(utils.drawMissingDataPlot(df)))
            doc.asis('<div style="clear:both;"></div>')
    with open(outputFile, 'w') as output:
        output.write(doc.getvalue())


def __drawVitalsSummary(df, colNames):

    with tag('table table-dark'):
        with tag('tr'):
            with tag('th'):
                text('Column')
            with tag('th'):
                text('DataType')
        for col in colNames:
            if col in df.columns:
                with tag('tr'):
                    with tag('td'):
                        text(col)
                    with tag('td'):
                        text(str(df[col].dtypes))


def __drawVitalsHistogram(df, col):

    fig, ax = plt.subplots()
    plt.xticks(rotation = 45)
    plt.hist(df[col])

    ax.set_title('Histogram - ' + col)
    ax.set_xlabel(col)
    ax.set_ylabel('Count')

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded


def __drawVitalsBoxplot(df, col):

    fig, ax = plt.subplots()
    plt.boxplot(df[~ np.isnan(df[col])][col])

    ax.set_title('Boxplot - ' + col)
    ax.set_ylabel(col)

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded


def __drawVitalsTable(df, col):

    with tag('table table-dark'):
        with tag('tr'):
            with tag('th'):
                text('Statistics - ' + col)
        with tag('tr'):
            with tag('td'):
                text('First Quartile')
            with tag('td'):
                text(str(round(df[col].quantile(0.25), 2)))
        with tag('tr'):
            with tag('td'):
                text('Mean')
            with tag('td'):
                text(str(round(df[col].mean(), 2)))
        with tag('tr'):
            with tag('td'):
                text('Median')
            with tag('td'):
                text(str(round(df[col].median(), 2)))
        with tag('tr'):
            with tag('td'):
                text('Mode')
            with tag('td'):
                text(str(round(df[col].mode()[0], 2)))
        with tag('tr'):
            with tag('td'):
                text('Third Quartile')
            with tag('td'):
                text(str(round(df[col].quantile(0.75), 2)))



import numpy as np
import pandas as pd

from sklearn.metrics import r2_score

import impyute as impy
import miceforest as mf
from statsmodels.multivariate.pca import PCA
from sklearn.impute import SimpleImputer
from sklearn.impute import KNNImputer
from sklearn.impute import SimpleImputer
from sklearn.decomposition import PCA as sklearnPCA
from utils import MissForest


def compare(fullDf, p=0.1):
    mask = np.random.choice(a=[True, False], size=fullDf.shape, p=[p, 1-p])
    missingDf = fullDf.mask(mask)

    meanImputer = SimpleImputer(strategy='mean')
    meanImputedData = meanImputer.fit_transform(missingDf)
    meanImputedDf = pd.DataFrame(meanImputedData, columns=[(col + '_mean') for col in missingDf.columns], index=missingDf.index)
    meanR2 = r2_score(fullDf, meanImputedDf)

    medianImputer = SimpleImputer(strategy='median')
    medianImputedData = medianImputer.fit_transform(missingDf)
    medianImputedDf = pd.DataFrame(medianImputedData, columns=[(col + '_median') for col in missingDf.columns], index=missingDf.index)
    medianR2 = r2_score(fullDf, medianImputedDf)

    knnImputer = KNNImputer()
    knnImputedData = knnImputer.fit_transform(missingDf)
    knnImputedDf = pd.DataFrame(knnImputedData, columns=[(col + '_knn') for col in missingDf.columns], index=missingDf.index)
    knnR2 = r2_score(fullDf, knnImputedDf)

    mfImputer = MissForest()
    mfImputedData = mfImputer.fit(missingDf).transform(missingDf)
    mfImputedDf = pd.DataFrame(mfImputedData, columns=[(col + '_mf') for col in missingDf.columns], index=missingDf.index)
    mfR2 = r2_score(fullDf, mfImputedDf)

    emImputedData = impy.em(missingDf.to_numpy())
    emImputedDataDf = pd.DataFrame(emImputedData, columns=[(col + '_em') for col in missingDf.columns], index=missingDf.index)
    emR2 = r2_score(fullDf, emImputedDataDf)

    miKernel = mf.ImputationKernel(
    missingDf,
    datasets=4,
    save_all_iterations=True,
    random_state=1
    )
    miKernel.mice(2)
    miImputedDataDf = miKernel.complete_data(dataset=0, inplace=False)
    miImputedDf = pd.DataFrame(miImputedDataDf.to_numpy(), columns=[(col + '_mi') for col in missingDf.columns], index=missingDf.index)
    miR2 = r2_score(fullDf, miImputedDf)

    return meanR2, medianR2, knnR2, mfR2, emR2, miR2


def impute(dataDf, algorithm):

    if algorithm == 'mean':
        meanImputer = SimpleImputer(strategy='mean')
        meanImputedData = meanImputer.fit_transform(dataDf)
        imputedDf = pd.DataFrame(meanImputedData, columns=[(col + '_mean') for col in dataDf.columns], index=dataDf.index)
    if algorithm == 'median':
        medianImputer = SimpleImputer(strategy='median')
        medianImputedData = medianImputer.fit_transform(dataDf)
        imputedDf = pd.DataFrame(medianImputedData, columns=[(col + '_median') for col in dataDf.columns], index=dataDf.index)
    if algorithm == 'knn':
        knnImputer = KNNImputer()
        knnImputedData = knnImputer.fit_transform(dataDf)
        imputedDf = pd.DataFrame(knnImputedData, columns=[(col + '_knn') for col in dataDf.columns], index=dataDf.index)
    if algorithm == 'miss_forest':
        mfImputer = MissForest()
        mfImputedData = mfImputer.fit(dataDf).transform(dataDf)
        imputedDf = pd.DataFrame(mfImputedData, columns=[(col + '_mf') for col in dataDf.columns], index=dataDf.index)
    if algorithm == 'expectation_maximization':
        emImputedData = impy.em(dataDf.to_numpy())
        imputedDf = pd.DataFrame(emImputedData, columns=[(col + '_em') for col in dataDf.columns], index=dataDf.index)
    if algorithm == 'multiple_imputation':
        miKernel = mf.ImputationKernel(
        dataDf,
        datasets=4,
        save_all_iterations=True,
        random_state=1
        )
        miKernel.mice(2)
        miImputedDataDf = miKernel.complete_data(dataset=0, inplace=False)
        imputedDf = pd.DataFrame(miImputedDataDf.to_numpy(), columns=[(col + '_mi') for col in dataDf.columns], index=dataDf.index)

    return imputedDf



import base64
import numpy as np
from io import BytesIO
from matplotlib import pyplot as plt
from yattag import Doc

from . import utils


doc, tag, text = Doc().tagtext()


def plot(
    df, 
    outputFile = 'demographics.html', 
    ageCol = 'age', 
    weightCol = 'weight', 
    heightCol = 'height', 
    genderCol = 'gender', 
    ethnicityCol = 'ethnicity', 
    dobCol = 'dob', 
    dodCol = 'dod'):

    doc.asis('<!DOCTYPE html>')
    with tag('html'):
        with tag('head'):
            doc.asis('<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.1/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-F3w7mX95PdgyTmZZMECAngseQB83DfGTowi0iMjiWaeVhAn4FJkqJByhZMI3AhiU" crossorigin="anonymous">')
        with tag('body'):
            doc.asis('<div style="clear:both;"></div>')
            with tag('div'):
                with tag('h1'):
                    doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                    with tag('span', klass='fs-4', style="margin: 10px;"):
                        text('Missing Data Plot')
                doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(utils.drawMissingDataPlot(df)))
            doc.asis('<div style="clear:both;"></div>')
            with tag('div'):
                with tag('h1'):
                    doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                    with tag('span', klass='fs-4', style="margin: 10px;"):
                        text('Age Distribution')
                with tag('div', klass='col-6', style="float: left;"):
                    doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(__drawAgeHistogram(df, ageCol)))
                with tag('div', klass='col-6', style="float: left;"):
                    doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(__drawAgeBoxplot(df, ageCol)))
            doc.asis('<div style="clear:both;"></div>')
            with tag('div'):
                with tag('h1'):
                    doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                    with tag('span', klass='fs-4', style="margin: 10px;"):
                        text('Weight Distribution')
                doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(__drawWeightHistogram(df, weightCol)))
            doc.asis('<div style="clear:both;"></div>')
            with tag('div'):
                with tag('h1'):
                    doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                    with tag('span', klass='fs-4', style="margin: 10px;"):
                        text('Height Distribution')
                doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(__drawHeightHistogram(df, heightCol)))
            doc.asis('<div style="clear:both;"></div>')
            with tag('div'):
                with tag('h1'):
                    doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                    with tag('span', klass='fs-4', style="margin: 10px;"):
                        text('Gender Value Counts')
                with tag('div', klass='col-6', style="float: left;"):
                    doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(__drawGenderBarplot(df, genderCol)))
                with tag('div', klass='col-6', style="float: left;"):
                    doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(__drawGenderPiechart(df, genderCol)))
            doc.asis('<div style="clear:both;"></div>')
            with tag('div'):
                with tag('h1'):
                    doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                    with tag('span', klass='fs-4', style="margin: 10px;"):
                        text('Ethnicity Value Counts')
                with tag('div', klass='col-6', style="float: left;"):
                    doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(__drawEthnicityBarplot(df, ethnicityCol)))
                with tag('div', klass='col-6', style="float: left;"):
                    doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(__drawEthnicityPiechart(df, ethnicityCol)))
            doc.asis('<div style="clear:both;"></div>')
            with tag('div'):
                with tag('h1'):
                    doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                    with tag('span', klass='fs-4', style="margin: 10px;"):
                        text('DOB Distribution')
                doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(__drawDobHistogram(df, dobCol)))
            doc.asis('<div style="clear:both;"></div>')
            with tag('div'):
                with tag('h1'):
                    doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                    with tag('span', klass='fs-4', style="margin: 10px;"):
                        text('DOD Distribution')
                doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(__drawDodHistogram(df, dodCol)))
    with open(outputFile, 'w') as output:
        output.write(doc.getvalue())


def __drawAgeHistogram(df, ageCol):

    fig, ax = plt.subplots()
    plt.xticks(rotation = 45)
    plt.hist(df[ageCol])

    ax.set_title('Age Histogram')
    ax.set_xlabel('Age')
    ax.set_ylabel('Count')

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded


def __drawAgeBoxplot(df, ageCol):

    fig, ax = plt.subplots()
    plt.boxplot(df[ageCol])

    ax.set_title('Age Boxplot')
    ax.set_ylabel('Age')

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded


def __drawWeightHistogram(df, weightCol):

    fig, ax = plt.subplots()
    plt.xticks(rotation = 45)
    plt.hist(df[weightCol])

    ax.set_title('Weight Distribution')
    ax.set_xlabel('Weight')
    ax.set_ylabel('Count')

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded


def __drawHeightHistogram(df, heightCol):

    fig, ax = plt.subplots()
    plt.xticks(rotation = 45)
    plt.hist(df[heightCol])

    ax.set_title('Height Distribution')
    ax.set_xlabel('Height')
    ax.set_ylabel('Count')

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded


def __drawGenderBarplot(df, genderCol):

    fig, ax = plt.subplots()
    plt.xticks(rotation = 45)
    df[genderCol].value_counts().plot(kind='barh')

    ax.set_title('Gender Barplot')
    ax.set_xlabel('Count')
    ax.set_ylabel('Gender')

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded


def __drawGenderPiechart(df, genderCol):

    fig, ax = plt.subplots()
    value_counts = df[genderCol].value_counts()
    labels = []
    values = []
    for key in value_counts.keys():
        labels.append(str(key) + ' (' + str(round(value_counts[key]/value_counts.sum()*100, 2)) + ' %)')
        values.append(value_counts[key])
    plt.pie(values, labels=labels)

    ax.set_title('Gender Piechart')

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded


def __drawEthnicityBarplot(df, ethnicityCol):

    fig, ax = plt.subplots()
    plt.xticks(rotation = 45)
    df[ethnicityCol].value_counts().plot(kind='barh')

    ax.set_title('Ethnicity Distribution')
    ax.set_xlabel('Count')
    ax.set_ylabel('Ethnicity')

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded


def __drawEthnicityPiechart(df, ethnicityCol):

    fig, ax = plt.subplots()
    value_counts = df[ethnicityCol].value_counts()
    labels = []
    values = []
    for key in value_counts.keys():
        labels.append(str(key) + ' (' + str(round(value_counts[key]/value_counts.sum()*100, 2)) + ' %)')
        values.append(value_counts[key])
    plt.pie(values, labels=labels)

    ax.set_title('Ethnicity Piechart')

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded


def __drawDobHistogram(df, dobCol):

    fig, ax = plt.subplots()
    plt.xticks(rotation = 45)
    plt.hist(df[dobCol])

    ax.set_title('DOB Distribution')
    ax.set_xlabel('DOB')
    ax.set_ylabel('Count')

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded


def __drawDodHistogram(df, dodCol):

    fig, ax = plt.subplots()
    plt.xticks(rotation = 45)
    plt.hist(df[dodCol])

    ax.set_title('DOD Distribution')
    ax.set_xlabel('DOD')
    ax.set_ylabel('Count')

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded



import base64
import itertools

import numpy as np
from io import BytesIO
from matplotlib import pyplot as plt
import seaborn as sns
from yattag import Doc

from anomaly import irt_ensemble


doc, tag, text = Doc().tagtext()


def plot(
    df,
    outputFile = 'lab_measurements_anomalies.html',
    glucoseCol = 'glucose',
    hemoglobinCol = 'hemoglobin',
    anion_gapCol = 'anion_gap',
    bicarbonateCol = 'bicarbonate',
    calcium_totalCol = 'calcium_total',
    chlorideCol = 'chloride',
    creatinineCol = 'creatinine',
    magnesiumCol = 'magnesium',
    phosphateCol = 'phosphate',
    potassiumCol = 'potassium',
    sodiumCol = 'sodium',
    urea_nitrogenCol = 'urea_nitrogen',
    hematocritCol = 'hematocrit',
    mchCol = 'mch',
    mchcCol = 'mchc',
    mcvCol = 'mcv',
    platelet_countCol = 'platelet_count',
    rdwCol = 'rdw',
    red_blood_cellsCol = 'red_blood_cells',
    white_blood_cellsCol = 'white_blood_cells'):

    colNames = [glucoseCol, hemoglobinCol, anion_gapCol, bicarbonateCol, calcium_totalCol, chlorideCol, creatinineCol, magnesiumCol, phosphateCol, potassiumCol, sodiumCol, urea_nitrogenCol, hematocritCol, mchCol, mchcCol, mcvCol, platelet_countCol, rdwCol, red_blood_cellsCol, white_blood_cellsCol]

    doc.asis('<!DOCTYPE html>')
    with tag('html'):
        doc.asis('<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.1/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-F3w7mX95PdgyTmZZMECAngseQB83DfGTowi0iMjiWaeVhAn4FJkqJByhZMI3AhiU" crossorigin="anonymous">')
        with tag('body'):
            for (x, y) in itertools.combinations(colNames, 2):
                if x in df.columns and y in df.columns:
                    with tag('div'):
                        with tag('h1'):
                            doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                            with tag('span', klass='fs-4', style="margin: 10px;"):
                                text('Anomaly plot - ' + x + ' - ' + y)
                        fig = __drawAnomalies(df, x, y)
                        if fig:
                            with tag('div', style="float: left;"):
                                doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(fig))
                doc.asis('<div style="clear:both;"></div>')
            doc.asis('<div style="clear:both;"></div>')
    with open(outputFile, 'w') as output:
        output.write(doc.getvalue())


def __drawAnomalies(df, x, y):

    anomalyDf = irt_ensemble(df[[x, y]])

    if anomalyDf is not None and len(anomalyDf) > 0:
        fig, ax = plt.subplots()
        sns.scatterplot(data=anomalyDf, x=x, y=y, hue='ensemble_scores')

        ax.set_title('Scatter plot')
        ax.set_xlabel(x)
        ax.set_ylabel(y)

        tempFile = BytesIO()
        fig.savefig(tempFile, format='png', bbox_inches='tight')
        encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')
    else:
        encoded = None

    return encoded


if __name__ == '__main__':
    import pandas as pd

    data = np.random.randn(100, 8)
    df = pd.DataFrame(data, columns=['glucose', 'hemoglobin', 'anion_gap', 'bicarbonate', 'calcium_total', 'chloride', 'creatinine', 'magnesium'])
    plot(df)



import base64
import itertools

import numpy as np
from io import BytesIO
from matplotlib import pyplot as plt
import seaborn as sns
from yattag import Doc

from anomaly import irt_ensemble


doc, tag, text = Doc().tagtext()


def plot(
    df,
    outputFile = 'vitals_anomalies.html',
    heartrateCol = 'heartrate',
    sysbpCol = 'sysbp',
    diabpCol = 'diabp',
    meanbpCol = 'meanbp',
    resprateCol = 'resprate',
    tempcCol = 'tempc',
    spo2Col = 'spo2',
    gcseyeCol = 'gcseye',
    gcsverbalCol = 'gcsverbal',
    gcsmotorCol = 'gcsmotor'):

    colNames = [heartrateCol, sysbpCol, diabpCol, meanbpCol, resprateCol, tempcCol, spo2Col, gcseyeCol, gcsverbalCol, gcsmotorCol]

    doc.asis('<!DOCTYPE html>')
    with tag('html'):
        doc.asis('<link href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.1/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-F3w7mX95PdgyTmZZMECAngseQB83DfGTowi0iMjiWaeVhAn4FJkqJByhZMI3AhiU" crossorigin="anonymous">')
        with tag('body'):
            for (x, y) in itertools.combinations(colNames, 2):
                if x in df.columns and y in df.columns:
                    with tag('div'):
                        with tag('h1'):
                            doc.asis('<svg xmlns="http://www.w3.org/2000/svg" width="25" height="25" fill="currentColor" class="bi bi-check-circle" viewBox="0 0 16 16"><path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/></svg>')
                            with tag('span', klass='fs-4', style="margin: 10px;"):
                                text('Anomaly plot - ' + x + ' - ' + y)
                        fig = __drawAnomalies(df, x, y)
                        if fig:
                            with tag('div', style="float: left;"):
                                doc.asis('<img src=\'data:image/png;base64,{}\'>'.format(fig))
                doc.asis('<div style="clear:both;"></div>')
            doc.asis('<div style="clear:both;"></div>')
    with open(outputFile, 'w') as output:
        output.write(doc.getvalue())


def __drawAnomalies(df, x, y):

    anomalyDf = irt_ensemble(df[[x, y]])

    if anomalyDf is not None and len(anomalyDf) > 0:
        fig, ax = plt.subplots()
        sns.scatterplot(data=anomalyDf, x=x, y=y, hue='ensemble_scores')

        ax.set_title('Scatter plot')
        ax.set_xlabel(x)
        ax.set_ylabel(y)

        tempFile = BytesIO()
        fig.savefig(tempFile, format='png', bbox_inches='tight')
        encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')
    else:
        encoded = None

    return encoded


if __name__ == '__main__':
    import pandas as pd
    import psycopg2

    # data = np.random.randn(100, 5)
    # df = pd.DataFrame(data, columns=['heartrate', 'sysbp', 'diabp', 'meanbp', 'resprateCol'])
    # plot(df)


    # information used to create a database connection
    sqluser = 'postgres'
    dbname = 'mimic4'
    hostname = 'localhost'
    port_number = 5434
    schema_name = 'mimiciv'

    # Connect to postgres with a copy of the MIMIC-III database
    con = psycopg2.connect(dbname=dbname, user=sqluser, host=hostname, port=port_number, password='mysecretpassword')

    # the below statement is prepended to queries to ensure they select from the right schema
    query_schema = 'set search_path to ' + schema_name + ';'

    query = query_schema + \
    """
    WITH vitals_stg_1 AS
    (
        SELECT icu.stay_id, cev.charttime
        , CASE
                WHEN itemid = 223761 THEN (cev.valuenum-32)/1.8
            ELSE cev.valuenum
        END AS valuenum
        , CASE
                WHEN itemid = 220045 THEN 'HEARTRATE'
                WHEN itemid = 220050 THEN 'SYSBP'
                WHEN itemid = 220179 THEN 'SYSBP'
                WHEN itemid = 220051 THEN 'DIASBP'
                WHEN itemid = 220180 THEN 'DIASBP'
                WHEN itemid = 220052 THEN 'MEANBP'
                WHEN itemid = 220181 THEN 'MEANBP'
                WHEN itemid = 225312 THEN 'MEANBP'
                WHEN itemid = 220210 THEN 'RESPRATE'
                WHEN itemid = 224688 THEN 'RESPRATE'
                WHEN itemid = 224689 THEN 'RESPRATE'
                WHEN itemid = 224690 THEN 'RESPRATE'
                WHEN itemid = 223761 THEN 'TEMPC'
                WHEN itemid = 223762 THEN 'TEMPC'
                WHEN itemid = 220277 THEN 'SPO2'
                WHEN itemid = 220739 THEN 'GCSEYE'
                WHEN itemid = 223900 THEN 'GCSVERBAL'
                WHEN itemid = 223901 THEN 'GCSMOTOR'
            ELSE null
            END AS label
        FROM mimiciv.icustays icu
        INNER JOIN mimiciv.chartevents cev
        ON cev.stay_id = icu.stay_id
        AND cev.charttime >= icu.intime
        AND cev.charttime <= icu.intime + interval '24 hour'
        WHERE cev.itemid IN
        (
        220045 -- heartrate
        , 220050, 220179 -- sysbp
        , 220051, 220180 -- diasbp
        , 220052, 220181, 225312 -- meanbp
        , 220210, 224688, 224689, 224690 -- resprate
        , 223761, 223762 -- tempc
        , 220277 -- SpO2
        , 220739 -- gcseye
        , 223900 -- gcsverbal
        , 223901 -- gscmotor
        )
        AND valuenum IS NOT null
    )
    , vitals_stg_2 AS
    (
    SELECT
        stay_id, valuenum, label
        , ROW_NUMBER() OVER (PARTITION BY stay_id, label ORDER BY charttime) AS rn
    FROM vitals_stg_1
    )
    , vitals_stg_3 AS
    (
    SELECT
        stay_id
        , rn
        , COALESCE(MAX(CASE WHEN label = 'HEARTRATE' THEN valuenum ELSE null END)) AS heartrate
        , COALESCE(MAX(CASE WHEN label = 'SYSBP' THEN valuenum ELSE null END)) AS sysbp
        , COALESCE(MAX(CASE WHEN label = 'DIASBP' THEN valuenum ELSE null END)) AS diabp
        , COALESCE(MAX(CASE WHEN label = 'MEANBP' THEN valuenum ELSE null END)) AS meanbp
        , COALESCE(MAX(CASE WHEN label = 'RESPRATE' THEN valuenum ELSE null END)) AS resprate
        , COALESCE(MAX(CASE WHEN label = 'TEMPC' THEN valuenum ELSE null END)) AS tempc
        , COALESCE(MAX(CASE WHEN label = 'SPO2' THEN valuenum ELSE null END)) AS spo2
        , COALESCE(MAX(CASE WHEN label = 'GCSEYE' THEN valuenum ELSE null END)) AS gcseye
        , COALESCE(MAX(CASE WHEN label = 'GCSVERBAL' THEN valuenum ELSE null END)) AS gcsverbal
        , COALESCE(MAX(CASE WHEN label = 'GCSMOTOR' THEN valuenum ELSE null END)) AS gcsmotor
    FROM vitals_stg_2
    GROUP BY stay_id, rn
    )
    , vitals_stg_4 AS
    (
        SELECT
        stay_id,
        AVG(heartrate) AS heartrate
        , AVG(sysbp) AS sysbp
        , AVG(diabp) AS diabp
        , AVG(meanbp) AS meanbp
        , AVG(resprate) AS resprate
        , AVG(tempc) AS tempc
        , AVG(spo2) AS spo2
        , AVG(gcseye) AS gcseye
        , AVG(gcsverbal) AS gcsverbal
        , AVG(gcsmotor) AS gcsmotor
        FROM vitals_stg_3
        GROUP BY stay_id
    )
    SELECT * FROM vitals_stg_4
    """

    vitals = pd.read_sql_query(query, con)

    vitals.fillna(0)

    plot(vitals)



import os, tempfile, subprocess
import pandas as pd


def irt_ensemble(data):

    inFile = tempfile.NamedTemporaryFile(delete=False)
    outFile = tempfile.NamedTemporaryFile(delete=False)
    try:
        data.to_csv(inFile, index=False)
        subprocess.call (["/usr/bin/Rscript", "--vanilla", "qc/script.r", inFile.name, outFile.name])

        if outFile:
            try:
                out = pd.read_csv(outFile)
            except:
                out = None
        else:
            out = None

    finally:
        inFile.close()
        outFile.close()
        os.unlink(inFile.name)
        os.unlink(outFile.name)

    return out


if __name__ == '__main__':
    import numpy as np
    data = np.random.randn(100, 2)
    df = pd.DataFrame(data, columns=['x', 'y'])

    out = irt_ensemble(df)
    print(out)



import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import qc.demographicsGraphs as demographicsGraphs
import qc.vitalsGraphs as vitalsGraphs
import qc.vitalsAnomalies as vitalsAnomalies
import qc.labMeasurementsGraphs as labMeasurementsGraphs
import qc.anomaly as anomaly



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='EHRQC',
    version='0.1',
    author='Yashpal Ramakrishnaiah',
    author_email='ryashpal.ramakrishnaiah1@monash.edu',
    description='Package for performing QC on Electronic Health Record (EHR) data',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/ryashpal/EHRQC',
    license='MIT',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],                                      # Information to filter the project on PyPi website
    python_requires='>=3.6',                # Minimum version requirement of the package
    py_modules=[
        "demographicsGraphs"
        , "vitalsGraphs"
        , "labMeasurementsGraphs"
        , "vitalsAnomalies"
        , "labMeasurementsAnomalies"
        , "missingDataImputation"
        , "anomaly"
        , "utils"
        ],             # Name of the python package
    package_dir={'qc':'qc'},     # Directory of the source code of the package
    install_requires=[
        ],
)



import base64
import numpy as np
from io import BytesIO
from matplotlib import pyplot as plt

import warnings

import numpy as np
from scipy.stats import mode

from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.utils.validation import check_is_fitted, check_array
from sklearn.ensemble import RandomForestClassifier, RandomForestRegressor



def drawMissingDataPlot(df):

    fig, ax = plt.subplots()
    plt.xticks(rotation = 45)

    cols = df.columns
    x_pos = np.arange(len(cols))

    plt.bar(x_pos, [df[col].isna().sum()/df.shape[0] for col in cols])

    ax.set_xticks(x_pos)
    ax.set_xticklabels(cols)

    ax.set_title('Missing Data Plot')
    ax.set_xlabel('Attribute')
    ax.set_ylabel('Missing Value Proportion')

    tempFile = BytesIO()
    fig.savefig(tempFile, format='png', bbox_inches='tight')
    encoded = base64.b64encode(tempFile.getvalue()).decode('utf-8')

    return encoded


def drawSummaryTable(df, tag, text, col):

    with tag('table table-dark'):
        with tag('tr'):
            with tag('th'):
                text('Statistics - ' + col)
        with tag('tr'):
            with tag('td'):
                text('First Quartile')
            with tag('td'):
                text(str(round(df[col].quantile(0.25), 2)))
        with tag('tr'):
            with tag('td'):
                text('Mean')
            with tag('td'):
                text(str(round(df[col].mean(), 2)))
        with tag('tr'):
            with tag('td'):
                text('Median')
            with tag('td'):
                text(str(round(df[col].median(), 2)))
        with tag('tr'):
            with tag('td'):
                text('Mode')
            with tag('td'):
                text(str(round(df[col].mode()[0], 2)))
        with tag('tr'):
            with tag('td'):
                text('Third Quartile')
            with tag('td'):
                text(str(round(df[col].quantile(0.75), 2)))
        with tag('tr'):
            with tag('td'):
                text('Standard Deviation')
            with tag('td'):
                text(str(round(df[col].std(), 2)))
        with tag('tr'):
            with tag('td'):
                text('Variance')
            with tag('td'):
                text(str(round(df[col].var(), 2)))
        with tag('tr'):
            with tag('td'):
                text('Skew')
            with tag('td'):
                text(str(round(df[col].skew(), 2)))
        with tag('tr'):
            with tag('td'):
                text('Kurtosis')
            with tag('td'):
                text(str(round(df[col].kurtosis(), 2)))


"""MissForest Imputer for Missing Data"""
# Author: Ashim Bhattarai
# License: GNU General Public License v3 (GPLv3)

# from .pairwise_external import _get_mask
def _get_mask(X, value_to_mask):
    """Compute the boolean mask X == missing_values."""
    if value_to_mask == "NaN" or np.isnan(value_to_mask):
        return np.isnan(X)
    else:
        return X == value_to_mask


__all__ = [
    'MissForest',
]


class MissForest(BaseEstimator, TransformerMixin):
    """Missing value imputation using Random Forests.
    MissForest imputes missing values using Random Forests in an iterative
    fashion. By default, the imputer begins imputing missing values of the
    column (which is expected to be a variable) with the smallest number of
    missing values -- let's call this the candidate column.
    The first step involves filling any missing values of the remaining,
    non-candidate, columns with an initial guess, which is the column mean for
    columns representing numerical variables and the column mode for columns
    representing categorical variables. After that, the imputer fits a random
    forest model with the candidate column as the outcome variable and the
    remaining columns as the predictors over all rows where the candidate
    column values are not missing.
    After the fit, the missing rows of the candidate column are
    imputed using the prediction from the fitted Random Forest. The
    rows of the non-candidate columns act as the input data for the fitted
    model.
    Following this, the imputer moves on to the next candidate column with the
    second smallest number of missing values from among the non-candidate
    columns in the first round. The process repeats itself for each column
    with a missing value, possibly over multiple iterations or epochs for
    each column, until the stopping criterion is met.
    The stopping criterion is governed by the "difference" between the imputed
    arrays over successive iterations. For numerical variables (num_vars_),
    the difference is defined as follows:
     sum((X_new[:, num_vars_] - X_old[:, num_vars_]) ** 2) /
     sum((X_new[:, num_vars_]) ** 2)
    For categorical variables(cat_vars_), the difference is defined as follows:
    sum(X_new[:, cat_vars_] != X_old[:, cat_vars_])) / n_cat_missing
    where X_new is the newly imputed array, X_old is the array imputed in the
    previous round, n_cat_missing is the total number of categorical
    values that are missing, and the sum() is performed both across rows
    and columns. Following [1], the stopping criterion is considered to have
    been met when difference between X_new and X_old increases for the first
    time for both types of variables (if available).
    Parameters
    ----------
    NOTE: Most parameter definitions below are taken verbatim from the
    Scikit-Learn documentation at [2] and [3].
    max_iter : int, optional (default = 10)
        The maximum iterations of the imputation process. Each column with a
        missing value is imputed exactly once in a given iteration.
    decreasing : boolean, optional (default = False)
        If set to True, columns are sorted according to decreasing number of
        missing values. In other words, imputation will move from imputing
        columns with the largest number of missing values to columns with
        fewest number of missing values.
    missing_values : np.nan, integer, optional (default = np.nan)
        The placeholder for the missing values. All occurrences of
        `missing_values` will be imputed.
    copy : boolean, optional (default = True)
        If True, a copy of X will be created. If False, imputation will
        be done in-place whenever possible.
    criterion : tuple, optional (default = ('mse', 'gini'))
        The function to measure the quality of a split.The first element of
        the tuple is for the Random Forest Regressor (for imputing numerical
        variables) while the second element is for the Random Forest
        Classifier (for imputing categorical variables).
    n_estimators : integer, optional (default=100)
        The number of trees in the forest.
    max_depth : integer or None, optional (default=None)
        The maximum depth of the tree. If None, then nodes are expanded until
        all leaves are pure or until all leaves contain less than
        min_samples_split samples.
    min_samples_split : int, float, optional (default=2)
        The minimum number of samples required to split an internal node:
        - If int, then consider `min_samples_split` as the minimum number.
        - If float, then `min_samples_split` is a fraction and
          `ceil(min_samples_split * n_samples)` are the minimum
          number of samples for each split.
    min_samples_leaf : int, float, optional (default=1)
        The minimum number of samples required to be at a leaf node.
        A split point at any depth will only be considered if it leaves at
        least ``min_samples_leaf`` training samples in each of the left and
        right branches.  This may have the effect of smoothing the model,
        especially in regression.
        - If int, then consider `min_samples_leaf` as the minimum number.
        - If float, then `min_samples_leaf` is a fraction and
          `ceil(min_samples_leaf * n_samples)` are the minimum
          number of samples for each node.
    min_weight_fraction_leaf : float, optional (default=0.)
        The minimum weighted fraction of the sum total of weights (of all
        the input samples) required to be at a leaf node. Samples have
        equal weight when sample_weight is not provided.
    max_features : int, float, string or None, optional (default="auto")
        The number of features to consider when looking for the best split:
        - If int, then consider `max_features` features at each split.
        - If float, then `max_features` is a fraction and
          `int(max_features * n_features)` features are considered at each
          split.
        - If "auto", then `max_features=sqrt(n_features)`.
        - If "sqrt", then `max_features=sqrt(n_features)` (same as "auto").
        - If "log2", then `max_features=log2(n_features)`.
        - If None, then `max_features=n_features`.
        Note: the search for a split does not stop until at least one
        valid partition of the node samples is found, even if it requires to
        effectively inspect more than ``max_features`` features.
    max_leaf_nodes : int or None, optional (default=None)
        Grow trees with ``max_leaf_nodes`` in best-first fashion.
        Best nodes are defined as relative reduction in impurity.
        If None then unlimited number of leaf nodes.
    min_impurity_decrease : float, optional (default=0.)
        A node will be split if this split induces a decrease of the impurity
        greater than or equal to this value.
        The weighted impurity decrease equation is the following::
            N_t / N * (impurity - N_t_R / N_t * right_impurity
                                - N_t_L / N_t * left_impurity)
        where ``N`` is the total number of samples, ``N_t`` is the number of
        samples at the current node, ``N_t_L`` is the number of samples in the
        left child, and ``N_t_R`` is the number of samples in the right child.
        ``N``, ``N_t``, ``N_t_R`` and ``N_t_L`` all refer to the weighted sum,
        if ``sample_weight`` is passed.
    bootstrap : boolean, optional (default=True)
        Whether bootstrap samples are used when building trees.
    oob_score : bool (default=False)
        Whether to use out-of-bag samples to estimate
        the generalization accuracy.
    n_jobs : int or None, optional (default=None)
        The number of jobs to run in parallel for both `fit` and `predict`.
        ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
        ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
        for more details.
    random_state : int, RandomState instance or None, optional (default=None)
        If int, random_state is the seed used by the random number generator;
        If RandomState instance, random_state is the random number generator;
        If None, the random number generator is the RandomState instance used
        by `np.random`.
    verbose : int, optional (default=0)
        Controls the verbosity when fitting and predicting.
    warm_start : bool, optional (default=False)
        When set to ``True``, reuse the solution of the previous call to fit
        and add more estimators to the ensemble, otherwise, just fit a whole
        new forest. See :term:`the Glossary <warm_start>`.
    class_weight : dict, list of dicts, "balanced", "balanced_subsample" or \
    None, optional (default=None)
        Weights associated with classes in the form ``{class_label: weight}``.
        If not given, all classes are supposed to have weight one. For
        multi-output problems, a list of dicts can be provided in the same
        order as the columns of y.
        Note that for multioutput (including multilabel) weights should be
        defined for each class of every column in its own dict. For example,
        for four-class multilabel classification weights should be
        [{0: 1, 1: 1}, {0: 1, 1: 5}, {0: 1, 1: 1}, {0: 1, 1: 1}] instead of
        [{1:1}, {2:5}, {3:1}, {4:1}].
        The "balanced" mode uses the values of y to automatically adjust
        weights inversely proportional to class frequencies in the input data
        as ``n_samples / (n_classes * np.bincount(y))``
        The "balanced_subsample" mode is the same as "balanced" except that
        weights are computed based on the bootstrap sample for every tree
        grown.
        For multi-output, the weights of each column of y will be multiplied.
        Note that these weights will be multiplied with sample_weight (passed
        through the fit method) if sample_weight is specified.
        NOTE: This parameter is only applicable for Random Forest Classifier
        objects (i.e., for categorical variables).
    Attributes
    ----------
    statistics_ : Dictionary of length two
        The first element is an array with the mean of each numerical feature
        being imputed while the second element is an array of modes of
        categorical features being imputed (if available, otherwise it
        will be None).
    References
    ----------
    * [1] Stekhoven, Daniel J., and Peter Bühlmann. "MissForest—non-parametric
      missing value imputation for mixed-type data." Bioinformatics 28.1
      (2011): 112-118.
    * [2] https://scikit-learn.org/stable/modules/generated/sklearn.ensemble.
      RandomForestRegressor.html#sklearn.ensemble.RandomForestRegressor
    * [3] https://scikit-learn.org/stable/modules/generated/sklearn.ensemble.
      RandomForestClassifier.html#sklearn.ensemble.RandomForestClassifier
    Examples
    --------
    >>> from missingpy import MissForest
    >>> nan = float("NaN")
    >>> X = [[1, 2, nan], [3, 4, 3], [nan, 6, 5], [8, 8, 7]]
    >>> imputer = MissForest(random_state=1337)
    >>> imputer.fit_transform(X)
    Iteration: 0
    Iteration: 1
    Iteration: 2
    array([[1.  , 2. , 3.92 ],
           [3.  , 4. , 3. ],
           [2.71, 6. , 5. ],
           [8.  , 8. , 7. ]])
    """

    def __init__(self, max_iter=10, decreasing=False, missing_values=np.nan,
                 copy=True, n_estimators=100, criterion=('squared_error', 'gini'),
                 max_depth=None, min_samples_split=2, min_samples_leaf=1,
                 min_weight_fraction_leaf=0.0, max_features='auto',
                 max_leaf_nodes=None, min_impurity_decrease=0.0,
                 bootstrap=True, oob_score=False, n_jobs=-1, random_state=None,
                 verbose=0, warm_start=False, class_weight=None):

        self.max_iter = max_iter
        self.decreasing = decreasing
        self.missing_values = missing_values
        self.copy = copy
        self.n_estimators = n_estimators
        self.criterion = criterion
        self.max_depth = max_depth
        self.min_samples_split = min_samples_split
        self.min_samples_leaf = min_samples_leaf
        self.min_weight_fraction_leaf = min_weight_fraction_leaf
        self.max_features = max_features
        self.max_leaf_nodes = max_leaf_nodes
        self.min_impurity_decrease = min_impurity_decrease
        self.bootstrap = bootstrap
        self.oob_score = oob_score
        self.n_jobs = n_jobs
        self.random_state = random_state
        self.verbose = verbose
        self.warm_start = warm_start
        self.class_weight = class_weight

    def _miss_forest(self, Ximp, mask):
        """The missForest algorithm"""

        # Count missing per column
        col_missing_count = mask.sum(axis=0)

        # Get col and row indices for missing
        missing_rows, missing_cols = np.where(mask)

        if self.num_vars_ is not None:
            # Only keep indices for numerical vars
            keep_idx_num = np.in1d(missing_cols, self.num_vars_)
            missing_num_rows = missing_rows[keep_idx_num]
            missing_num_cols = missing_cols[keep_idx_num]

            # Make initial guess for missing values
            col_means = np.full(Ximp.shape[1], fill_value=np.nan)
            col_means[self.num_vars_] = self.statistics_.get('col_means')
            Ximp[missing_num_rows, missing_num_cols] = np.take(
                col_means, missing_num_cols)

            # Reg criterion
            reg_criterion = self.criterion if type(self.criterion) == str \
                else self.criterion[0]

            # Instantiate regression model
            rf_regressor = RandomForestRegressor(
                n_estimators=self.n_estimators,
                criterion=reg_criterion,
                max_depth=self.max_depth,
                min_samples_split=self.min_samples_split,
                min_samples_leaf=self.min_samples_leaf,
                min_weight_fraction_leaf=self.min_weight_fraction_leaf,
                max_features=self.max_features,
                max_leaf_nodes=self.max_leaf_nodes,
                min_impurity_decrease=self.min_impurity_decrease,
                bootstrap=self.bootstrap,
                oob_score=self.oob_score,
                n_jobs=self.n_jobs,
                random_state=self.random_state,
                verbose=self.verbose,
                warm_start=self.warm_start)

        # If needed, repeat for categorical variables
        if self.cat_vars_ is not None:
            # Calculate total number of missing categorical values (used later)
            n_catmissing = np.sum(mask[:, self.cat_vars_])

            # Only keep indices for categorical vars
            keep_idx_cat = np.in1d(missing_cols, self.cat_vars_)
            missing_cat_rows = missing_rows[keep_idx_cat]
            missing_cat_cols = missing_cols[keep_idx_cat]

            # Make initial guess for missing values
            col_modes = np.full(Ximp.shape[1], fill_value=np.nan)
            col_modes[self.cat_vars_] = self.statistics_.get('col_modes')
            Ximp[missing_cat_rows, missing_cat_cols] = np.take(col_modes, missing_cat_cols)

            # Classfication criterion
            clf_criterion = self.criterion if type(self.criterion) == str \
                else self.criterion[1]

            # Instantiate classification model
            rf_classifier = RandomForestClassifier(
                n_estimators=self.n_estimators,
                criterion=clf_criterion,
                max_depth=self.max_depth,
                min_samples_split=self.min_samples_split,
                min_samples_leaf=self.min_samples_leaf,
                min_weight_fraction_leaf=self.min_weight_fraction_leaf,
                max_features=self.max_features,
                max_leaf_nodes=self.max_leaf_nodes,
                min_impurity_decrease=self.min_impurity_decrease,
                bootstrap=self.bootstrap,
                oob_score=self.oob_score,
                n_jobs=self.n_jobs,
                random_state=self.random_state,
                verbose=self.verbose,
                warm_start=self.warm_start,
                class_weight=self.class_weight)

        # 2. misscount_idx: sorted indices of cols in X based on missing count
        misscount_idx = np.argsort(col_missing_count)
        # Reverse order if decreasing is set to True
        if self.decreasing is True:
            misscount_idx = misscount_idx[::-1]

        # 3. While new_gammas < old_gammas & self.iter_count_ < max_iter loop:
        self.iter_count_ = 0
        gamma_new = 0
        gamma_old = np.inf
        gamma_newcat = 0
        gamma_oldcat = np.inf
        col_index = np.arange(Ximp.shape[1])

        while (
                gamma_new < gamma_old or gamma_newcat < gamma_oldcat) and \
                self.iter_count_ < self.max_iter:

            # 4. store previously imputed matrix
            Ximp_old = np.copy(Ximp)
            if self.iter_count_ != 0:
                gamma_old = gamma_new
                gamma_oldcat = gamma_newcat
            # 5. loop
            for s in misscount_idx:
                # Column indices other than the one being imputed
                s_prime = np.delete(col_index, s)

                # Get indices of rows where 's' is observed and missing
                obs_rows = np.where(~mask[:, s])[0]
                mis_rows = np.where(mask[:, s])[0]

                # If no missing, then skip
                if len(mis_rows) == 0:
                    continue

                # Get observed values of 's'
                yobs = Ximp[obs_rows, s]

                # Get 'X' for both observed and missing 's' column
                xobs = Ximp[np.ix_(obs_rows, s_prime)]
                xmis = Ximp[np.ix_(mis_rows, s_prime)]

                # 6. Fit a random forest over observed and predict the missing
                if self.cat_vars_ is not None and s in self.cat_vars_:
                    rf_classifier.fit(X=xobs, y=yobs)
                    # 7. predict ymis(s) using xmis(x)
                    ymis = rf_classifier.predict(xmis)
                    # 8. update imputed matrix using predicted matrix ymis(s)
                    Ximp[mis_rows, s] = ymis
                else:
                    rf_regressor.fit(X=xobs, y=yobs)
                    # 7. predict ymis(s) using xmis(x)
                    ymis = rf_regressor.predict(xmis)
                    # 8. update imputed matrix using predicted matrix ymis(s)
                    Ximp[mis_rows, s] = ymis

            # 9. Update gamma (stopping criterion)
            if self.cat_vars_ is not None:
                gamma_newcat = np.sum(
                    (Ximp[:, self.cat_vars_] != Ximp_old[:, self.cat_vars_])) / n_catmissing
            if self.num_vars_ is not None:
                gamma_new = np.sum((Ximp[:, self.num_vars_] - Ximp_old[:, self.num_vars_]) ** 2) / np.sum((Ximp[:, self.num_vars_]) ** 2)

            print("Iteration:", self.iter_count_)
            self.iter_count_ += 1

        return Ximp_old

    def fit(self, X, y=None, cat_vars=None):
        """Fit the imputer on X.
        Parameters
        ----------
        X : {array-like}, shape (n_samples, n_features)
            Input data, where ``n_samples`` is the number of samples and
            ``n_features`` is the number of features.
        cat_vars : int or array of ints, optional (default = None)
            An int or an array containing column indices of categorical
            variable(s)/feature(s) present in the dataset X.
            ``None`` if there are no categorical variables in the dataset.
        Returns
        -------
        self : object
            Returns self.
        """

        # Check data integrity and calling arguments
        force_all_finite = False if self.missing_values in ["NaN",
                                                            np.nan] else True

        X = check_array(X, accept_sparse=False, dtype=np.float64,
                        force_all_finite=force_all_finite, copy=self.copy)

        # Check for +/- inf
        if np.any(np.isinf(X)):
            raise ValueError("+/- inf values are not supported.")

        # Check if any column has all missing
        mask = _get_mask(X, self.missing_values)
        if np.any(mask.sum(axis=0) >= (X.shape[0])):
            raise ValueError("One or more columns have all rows missing.")

        # Check cat_vars type and convert if necessary
        if cat_vars is not None:
            if type(cat_vars) == int:
                cat_vars = [cat_vars]
            elif type(cat_vars) == list or type(cat_vars) == np.ndarray:
                if np.array(cat_vars).dtype != int:
                    raise ValueError(
                        "cat_vars needs to be either an int or an array "
                        "of ints.")
            else:
                raise ValueError("cat_vars needs to be either an int or an array "
                                 "of ints.")

        # Identify numerical variables
        num_vars = np.setdiff1d(np.arange(X.shape[1]), cat_vars)
        num_vars = num_vars if len(num_vars) > 0 else None

        # First replace missing values with NaN if it is something else
        if self.missing_values not in ['NaN', np.nan]:
            X[np.where(X == self.missing_values)] = np.nan

        # Now, make initial guess for missing values
        col_means = np.nanmean(X[:, num_vars], axis=0) if num_vars is not None else None
        col_modes = mode(
            X[:, cat_vars], axis=0, nan_policy='omit')[0] if cat_vars is not \
                                                           None else None

        self.cat_vars_ = cat_vars
        self.num_vars_ = num_vars
        self.statistics_ = {"col_means": col_means, "col_modes": col_modes}

        return self

    def transform(self, X):
        """Impute all missing values in X.
        Parameters
        ----------
        X : {array-like}, shape = [n_samples, n_features]
            The input data to complete.
        Returns
        -------
        X : {array-like}, shape = [n_samples, n_features]
            The imputed dataset.
        """
        # Confirm whether fit() has been called
        check_is_fitted(self, ["cat_vars_", "num_vars_", "statistics_"])

        # Check data integrity
        force_all_finite = False if self.missing_values in ["NaN",
                                                            np.nan] else True
        X = check_array(X, accept_sparse=False, dtype=np.float64,
                        force_all_finite=force_all_finite, copy=self.copy)

        # Check for +/- inf
        if np.any(np.isinf(X)):
            raise ValueError("+/- inf values are not supported.")

        # Check if any column has all missing
        mask = _get_mask(X, self.missing_values)
        if np.any(mask.sum(axis=0) >= (X.shape[0])):
            raise ValueError("One or more columns have all rows missing.")

        # Get fitted X col count and ensure correct dimension
        n_cols_fit_X = (0 if self.num_vars_ is None else len(self.num_vars_)) \
            + (0 if self.cat_vars_ is None else len(self.cat_vars_))
        _, n_cols_X = X.shape

        if n_cols_X != n_cols_fit_X:
            raise ValueError("Incompatible dimension between the fitted "
                             "dataset and the one to be transformed.")

        # Check if anything is actually missing and if not return original X                             
        mask = _get_mask(X, self.missing_values)
        if not mask.sum() > 0:
            warnings.warn("No missing value located; returning original "
                          "dataset.")
            return X

        # row_total_missing = mask.sum(axis=1)
        # if not np.any(row_total_missing):
        #     return X

        # Call missForest function to impute missing
        X = self._miss_forest(X, mask)

        # Return imputed dataset
        return X

    def fit_transform(self, X, y=None, **fit_params):
        """Fit MissForest and impute all missing values in X.
        Parameters
        ----------
        X : {array-like}, shape (n_samples, n_features)
            Input data, where ``n_samples`` is the number of samples and
            ``n_features`` is the number of features.
        Returns
        -------
        X : {array-like}, shape (n_samples, n_features)
            Returns imputed dataset.
        """
        return self.fit(X, **fit_params).transform(X)


import os
import psycopg2
import pandas as pd

def _getDemographics():
    # information used to create a database connection
    sqluser = 'postgres'
    dbname = 'mimic4'
    hostname = 'localhost'
    port_number = 5434
    schema_name = 'mimiciv'

    # Connect to postgres with a copy of the MIMIC-III database
    con = psycopg2.connect(dbname=dbname, user=sqluser, host=hostname, port=port_number, password='mysecretpassword')

    # the below statement is prepended to queries to ensure they select from the right schema
    query_schema = 'set search_path to ' + schema_name + ';'

    query = query_schema + \
    """
    WITH ht AS
    (
    SELECT 
        c.subject_id, c.stay_id, c.charttime,
        -- Ensure that all heights are in centimeters, and fix data as needed
        CASE
            -- rule for neonates
            WHEN pt.anchor_age = 0
            AND (c.valuenum * 2.54) < 80
            THEN c.valuenum * 2.54
            -- rule for adults
            WHEN pt.anchor_age > 0
            AND (c.valuenum * 2.54) > 120
            AND (c.valuenum * 2.54) < 230
            THEN c.valuenum * 2.54
            -- set bad data to NULL
            ELSE NULL
        END AS height
        , ROW_NUMBER() OVER (PARTITION BY stay_id ORDER BY charttime DESC) AS rn
    FROM mimiciv.chartevents c
    INNER JOIN mimiciv.patients pt
        ON c.subject_id = pt.subject_id
    WHERE c.valuenum IS NOT NULL
    AND c.valuenum != 0
    AND c.itemid IN
    (
        226707 -- Height (measured in inches)
        -- note we intentionally ignore the below ITEMID in metavision
        -- these are duplicate data in a different unit
        -- , 226730 -- Height (cm)
    )
    )
    , wt AS
    (
        SELECT
            c.stay_id
        , c.charttime
        -- TODO: eliminate obvious outliers if there is a reasonable weight
        , c.valuenum as weight
        , ROW_NUMBER() OVER (PARTITION BY stay_id ORDER BY charttime DESC) AS rn
        FROM mimiciv.chartevents c
        WHERE c.valuenum IS NOT NULL
        AND c.itemid = 226512 -- Admit Wt
        AND c.stay_id IS NOT NULL
        AND c.valuenum > 0
    )
    SELECT
    ie.subject_id, ie.hadm_id, ie.stay_id
    , pat.gender AS gender
    , FLOOR(DATE_PART('day', adm.admittime - make_timestamp(pat.anchor_year, 1, 1, 0, 0, 0))/365.0) + pat.anchor_age as age
    , adm.ethnicity AS ethnicity
    , ht.height as height
    , wt.weight as weight
    , make_timestamp(pat.anchor_year, 1, 1, 0, 0, 0) as dob
    , pat.dod as dod
    FROM mimiciv.icustays ie
    INNER JOIN mimiciv.admissions adm
    ON ie.hadm_id = adm.hadm_id
    INNER JOIN mimiciv.patients pat
    ON ie.subject_id = pat.subject_id
    LEFT JOIN ht
    ON ie.stay_id = ht.stay_id AND ht.rn = 1
    LEFT JOIN wt
    ON ie.stay_id = wt.stay_id AND wt.rn = 1
    """

    static = pd.read_sql_query(query, con)

    return static


def _getVitals():
    # information used to create a database connection
    sqluser = 'postgres'
    dbname = 'mimic4'
    hostname = 'localhost'
    port_number = 5434
    schema_name = 'mimiciv'

    # Connect to postgres with a copy of the MIMIC-III database
    con = psycopg2.connect(dbname=dbname, user=sqluser, host=hostname, port=port_number, password='mysecretpassword')

    # the below statement is prepended to queries to ensure they select from the right schema
    query_schema = 'set search_path to ' + schema_name + ';'

    query = query_schema + \
    """
    WITH vitals_stg_1 AS
    (
        SELECT icu.stay_id, cev.charttime
        , CASE
                WHEN itemid = 223761 THEN (cev.valuenum-32)/1.8
            ELSE cev.valuenum
        END AS valuenum
        , CASE
                WHEN itemid = 220045 THEN 'HEARTRATE'
                WHEN itemid = 220050 THEN 'SYSBP'
                WHEN itemid = 220179 THEN 'SYSBP'
                WHEN itemid = 220051 THEN 'DIASBP'
                WHEN itemid = 220180 THEN 'DIASBP'
                WHEN itemid = 220052 THEN 'MEANBP'
                WHEN itemid = 220181 THEN 'MEANBP'
                WHEN itemid = 225312 THEN 'MEANBP'
                WHEN itemid = 220210 THEN 'RESPRATE'
                WHEN itemid = 224688 THEN 'RESPRATE'
                WHEN itemid = 224689 THEN 'RESPRATE'
                WHEN itemid = 224690 THEN 'RESPRATE'
                WHEN itemid = 223761 THEN 'TEMPC'
                WHEN itemid = 223762 THEN 'TEMPC'
                WHEN itemid = 220277 THEN 'SPO2'
                WHEN itemid = 220739 THEN 'GCSEYE'
                WHEN itemid = 223900 THEN 'GCSVERBAL'
                WHEN itemid = 223901 THEN 'GCSMOTOR'
            ELSE null
            END AS label
        FROM mimiciv.icustays icu
        INNER JOIN mimiciv.chartevents cev
        ON cev.stay_id = icu.stay_id
        AND cev.charttime >= icu.intime
        AND cev.charttime <= icu.intime + interval '24 hour'
        WHERE cev.itemid IN
        (
        220045 -- heartrate
        , 220050, 220179 -- sysbp
        , 220051, 220180 -- diasbp
        , 220052, 220181, 225312 -- meanbp
        , 220210, 224688, 224689, 224690 -- resprate
        , 223761, 223762 -- tempc
        , 220277 -- SpO2
        , 220739 -- gcseye
        , 223900 -- gcsverbal
        , 223901 -- gscmotor
        )
        AND valuenum IS NOT null
    )
    , vitals_stg_2 AS
    (
    SELECT
        stay_id, valuenum, label
        , ROW_NUMBER() OVER (PARTITION BY stay_id, label ORDER BY charttime) AS rn
    FROM vitals_stg_1
    )
    , vitals_stg_3 AS
    (
    SELECT
        stay_id
        , rn
        , COALESCE(MAX(CASE WHEN label = 'HEARTRATE' THEN valuenum ELSE null END)) AS heartrate
        , COALESCE(MAX(CASE WHEN label = 'SYSBP' THEN valuenum ELSE null END)) AS sysbp
        , COALESCE(MAX(CASE WHEN label = 'DIASBP' THEN valuenum ELSE null END)) AS diabp
        , COALESCE(MAX(CASE WHEN label = 'MEANBP' THEN valuenum ELSE null END)) AS meanbp
        , COALESCE(MAX(CASE WHEN label = 'RESPRATE' THEN valuenum ELSE null END)) AS resprate
        , COALESCE(MAX(CASE WHEN label = 'TEMPC' THEN valuenum ELSE null END)) AS tempc
        , COALESCE(MAX(CASE WHEN label = 'SPO2' THEN valuenum ELSE null END)) AS spo2
        , COALESCE(MAX(CASE WHEN label = 'GCSEYE' THEN valuenum ELSE null END)) AS gcseye
        , COALESCE(MAX(CASE WHEN label = 'GCSVERBAL' THEN valuenum ELSE null END)) AS gcsverbal
        , COALESCE(MAX(CASE WHEN label = 'GCSMOTOR' THEN valuenum ELSE null END)) AS gcsmotor
    FROM vitals_stg_2
    GROUP BY stay_id, rn
    )
    , vitals_stg_4 AS
    (
        SELECT
        stay_id,
        AVG(heartrate) AS heartrate
        , AVG(sysbp) AS sysbp
        , AVG(diabp) AS diabp
        , AVG(meanbp) AS meanbp
        , AVG(resprate) AS resprate
        , AVG(tempc) AS tempc
        , AVG(spo2) AS spo2
        , AVG(gcseye) AS gcseye
        , AVG(gcsverbal) AS gcsverbal
        , AVG(gcsmotor) AS gcsmotor
        FROM vitals_stg_3
        GROUP BY stay_id
    )
    SELECT * FROM vitals_stg_4
    """

    vitals = pd.read_sql_query(query, con)

    return vitals


def _getLabMeasurements():
    # information used to create a database connection
    sqluser = 'postgres'
    dbname = 'mimic4'
    hostname = 'localhost'
    port_number = 5434

    # Connect to postgres with a copy of the MIMIC-III database
    con = psycopg2.connect(dbname=dbname, user=sqluser, host=hostname, port=port_number, password='mysecretpassword')

    curDir = os.path.dirname(__file__)
    mimicOmopSelectedLabMeasurementsPath = os.path.join(curDir, '../extract/sql/mimic_omop_selected_lab_measurements.sql')
    mimicOmopSelectedLabMeasurementsFile = open(mimicOmopSelectedLabMeasurementsPath)
    mimicOmopSelectedLabMeasurementsDf = pd.read_sql_query(mimicOmopSelectedLabMeasurementsFile.read(), con)
    return mimicOmopSelectedLabMeasurementsDf
